@@ -2,7 +2,9 @@ use std::convert::TryFrom;
 use std::ffi::CString;
 use std::ptr;
 
-use crate::{check_status, sys, Callback, Error, Result, Status};
+use crate::{
+  bindgen_runtime::TypeName, check_status, sys, type_of, Callback, Error, Result, Status, ValueType,
+};
 
 #[cfg(feature = "serde-json")]
 mod de;
@@ -17,7 +19,6 @@ mod buffer;
 #[cfg(feature = "napi5")]
 mod date;
 mod either;
-mod error;
 mod escapable_handle_scope;
 mod function;
 mod global;
@@ -29,11 +30,10 @@ mod tagged_object;
 mod undefined;
 mod value;
 mod value_ref;
-mod value_type;
 
 pub use arraybuffer::*;
 #[cfg(feature = "napi6")]
-pub use bigint::JsBigint;
+pub use bigint::JsBigInt;
 pub use boolean::JsBoolean;
 pub use buffer::*;
 #[cfg(feature = "napi5")]
@@ -41,7 +41,6 @@ pub use date::*;
 #[cfg(feature = "serde-json")]
 pub(crate) use de::De;
 pub use either::Either;
-pub use error::*;
 pub use escapable_handle_scope::EscapableHandleScope;
 pub use function::JsFunction;
 pub use global::*;
@@ -55,7 +54,6 @@ pub(crate) use tagged_object::TaggedObject;
 pub use undefined::JsUndefined;
 pub(crate) use value::Value;
 pub use value_ref::*;
-pub use value_type::ValueType;
 
 // Value types
 
@@ -64,22 +62,39 @@ pub struct JsUnknown(pub(crate) Value);
 #[derive(Clone, Copy)]
 pub struct JsNull(pub(crate) Value);
 
+impl TypeName for JsNull {
+  fn type_name() -> &'static str {
+    "null"
+  }
+
+  fn value_type() -> ValueType {
+    ValueType::Null
+  }
+}
+
 #[derive(Clone, Copy)]
 pub struct JsSymbol(pub(crate) Value);
 
+impl TypeName for JsSymbol {
+  fn type_name() -> &'static str {
+    "symbol"
+  }
+
+  fn value_type() -> ValueType {
+    ValueType::Symbol
+  }
+}
+
 pub struct JsExternal(pub(crate) Value);
 
-#[doc(hidden)]
-#[macro_export(local_inner_macros)]
-macro_rules! type_of {
-  ($env:expr, $value:expr) => {{
-    use std::convert::TryFrom;
-    use $crate::sys;
-    let mut value_type = 0;
-    let status = sys::napi_typeof($env, $value, &mut value_type);
-    check_status!(status)
-      .and_then(|_| ValueType::try_from(value_type).or_else(|_| Ok(ValueType::Unknown)))
-  }};
+impl TypeName for JsExternal {
+  fn type_name() -> &'static str {
+    "external"
+  }
+
+  fn value_type() -> ValueType {
+    ValueType::External
+  }
 }
 
 macro_rules! impl_napi_value_trait {
@@ -134,12 +149,22 @@ macro_rules! impl_napi_value_trait {
 macro_rules! impl_js_value_methods {
   ($js_value:ident) => {
     impl $js_value {
-      #[inline]
       pub fn into_unknown(self) -> JsUnknown {
         unsafe { JsUnknown::from_raw_unchecked(self.0.env, self.0.value) }
       }
 
-      #[inline]
+      pub fn coerce_to_bool(self) -> Result<JsBoolean> {
+        let mut new_raw_value = ptr::null_mut();
+        check_status!(unsafe {
+          sys::napi_coerce_to_bool(self.0.env, self.0.value, &mut new_raw_value)
+        })?;
+        Ok(JsBoolean(Value {
+          env: self.0.env,
+          value: new_raw_value,
+          value_type: ValueType::Boolean,
+        }))
+      }
+
       pub fn coerce_to_number(self) -> Result<JsNumber> {
         let mut new_raw_value = ptr::null_mut();
         check_status!(unsafe {
@@ -152,7 +177,6 @@ macro_rules! impl_js_value_methods {
         }))
       }
 
-      #[inline]
       pub fn coerce_to_string(self) -> Result<JsString> {
         let mut new_raw_value = ptr::null_mut();
         check_status!(unsafe {
@@ -165,7 +189,6 @@ macro_rules! impl_js_value_methods {
         }))
       }
 
-      #[inline]
       pub fn coerce_to_object(self) -> Result<JsObject> {
         let mut new_raw_value = ptr::null_mut();
         check_status!(unsafe {
@@ -179,56 +202,48 @@ macro_rules! impl_js_value_methods {
       }
 
       #[cfg(feature = "napi5")]
-      #[inline]
       pub fn is_date(&self) -> Result<bool> {
         let mut is_date = true;
         check_status!(unsafe { sys::napi_is_date(self.0.env, self.0.value, &mut is_date) })?;
         Ok(is_date)
       }
 
-      #[inline]
       pub fn is_promise(&self) -> Result<bool> {
         let mut is_promise = true;
         check_status!(unsafe { sys::napi_is_promise(self.0.env, self.0.value, &mut is_promise) })?;
         Ok(is_promise)
       }
 
-      #[inline]
       pub fn is_error(&self) -> Result<bool> {
         let mut result = false;
         check_status!(unsafe { sys::napi_is_error(self.0.env, self.0.value, &mut result) })?;
         Ok(result)
       }
 
-      #[inline]
       pub fn is_typedarray(&self) -> Result<bool> {
         let mut result = false;
         check_status!(unsafe { sys::napi_is_typedarray(self.0.env, self.0.value, &mut result) })?;
         Ok(result)
       }
 
-      #[inline]
       pub fn is_dataview(&self) -> Result<bool> {
         let mut result = false;
         check_status!(unsafe { sys::napi_is_dataview(self.0.env, self.0.value, &mut result) })?;
         Ok(result)
       }
 
-      #[inline]
       pub fn is_array(&self) -> Result<bool> {
         let mut is_array = false;
         check_status!(unsafe { sys::napi_is_array(self.0.env, self.0.value, &mut is_array) })?;
         Ok(is_array)
       }
 
-      #[inline]
       pub fn is_buffer(&self) -> Result<bool> {
         let mut is_buffer = false;
         check_status!(unsafe { sys::napi_is_buffer(self.0.env, self.0.value, &mut is_buffer) })?;
         Ok(is_buffer)
       }
 
-      #[inline]
       pub fn instanceof<Constructor>(&self, constructor: Constructor) -> Result<bool>
       where
         Constructor: NapiRaw,
@@ -239,18 +254,6 @@ macro_rules! impl_js_value_methods {
         })?;
         Ok(result)
       }
-
-      #[cfg(feature = "napi8")]
-      #[inline]
-      pub fn freeze(&mut self) -> Result<()> {
-        check_status!(unsafe { sys::napi_object_freeze(self.0.env, self.0.value) })
-      }
-
-      #[cfg(feature = "napi8")]
-      #[inline]
-      pub fn seal(&mut self) -> Result<()> {
-        check_status!(unsafe { sys::napi_object_seal(self.0.env, self.0.value) })
-      }
     }
   };
 }
@@ -258,18 +261,16 @@ macro_rules! impl_js_value_methods {
 macro_rules! impl_object_methods {
   ($js_value:ident) => {
     impl $js_value {
-      #[inline]
-      pub fn set_property<V, K>(&mut self, key: K, value: V) -> Result<()>
+      pub fn set_property<K, V>(&mut self, key: K, value: V) -> Result<()>
       where
-        V: NapiRaw,
         K: NapiRaw,
+        V: NapiRaw,
       {
         check_status!(unsafe {
           sys::napi_set_property(self.0.env, self.0.value, key.raw(), value.raw())
         })
       }
 
-      #[inline]
       pub fn get_property<K, T>(&self, key: K) -> Result<T>
       where
         K: NapiRaw,
@@ -282,7 +283,6 @@ macro_rules! impl_object_methods {
         unsafe { T::from_raw(self.0.env, raw_value) }
       }
 
-      #[inline]
       pub fn get_property_unchecked<K, T>(&self, key: K) -> Result<T>
       where
         K: NapiRaw,
@@ -295,7 +295,6 @@ macro_rules! impl_object_methods {
         Ok(unsafe { T::from_raw_unchecked(self.0.env, raw_value) })
       }
 
-      #[inline]
       pub fn set_named_property<T>(&mut self, name: &str, value: T) -> Result<()>
       where
         T: NapiRaw,
@@ -306,7 +305,6 @@ macro_rules! impl_object_methods {
         })
       }
 
-      #[inline]
       pub fn create_named_method(&mut self, name: &str, function: Callback) -> Result<()> {
         let mut js_function = ptr::null_mut();
         let len = name.len();
@@ -326,7 +324,6 @@ macro_rules! impl_object_methods {
         })
       }
 
-      #[inline]
       pub fn get_named_property<T>(&self, name: &str) -> Result<T>
       where
         T: NapiValue,
@@ -339,7 +336,6 @@ macro_rules! impl_object_methods {
         unsafe { T::from_raw(self.0.env, raw_value) }
       }
 
-      #[inline]
       pub fn get_named_property_unchecked<T>(&self, name: &str) -> Result<T>
       where
         T: NapiValue,
@@ -352,7 +348,6 @@ macro_rules! impl_object_methods {
         Ok(unsafe { T::from_raw_unchecked(self.0.env, raw_value) })
       }
 
-      #[inline]
       pub fn has_named_property(&self, name: &str) -> Result<bool> {
         let mut result = false;
         let key = CString::new(name)?;
@@ -362,7 +357,6 @@ macro_rules! impl_object_methods {
         Ok(result)
       }
 
-      #[inline]
       pub fn delete_property<S>(&mut self, name: S) -> Result<bool>
       where
         S: NapiRaw,
@@ -374,7 +368,6 @@ macro_rules! impl_object_methods {
         Ok(result)
       }
 
-      #[inline]
       pub fn delete_named_property(&mut self, name: &str) -> Result<bool> {
         let mut result = false;
         let key_str = CString::new(name)?;
@@ -388,7 +381,6 @@ macro_rules! impl_object_methods {
         Ok(result)
       }
 
-      #[inline]
       pub fn has_own_property(&self, key: &str) -> Result<bool> {
         let mut result = false;
         let string = CString::new(key)?;
@@ -402,7 +394,6 @@ macro_rules! impl_object_methods {
         Ok(result)
       }
 
-      #[inline]
       pub fn has_own_property_js<K>(&self, key: K) -> Result<bool>
       where
         K: NapiRaw,
@@ -414,7 +405,6 @@ macro_rules! impl_object_methods {
         Ok(result)
       }
 
-      #[inline]
       pub fn has_property(&self, name: &str) -> Result<bool> {
         let string = CString::new(name)?;
         let mut js_key = ptr::null_mut();
@@ -428,7 +418,6 @@ macro_rules! impl_object_methods {
         Ok(result)
       }
 
-      #[inline]
       pub fn has_property_js<K>(&self, name: K) -> Result<bool>
       where
         K: NapiRaw,
@@ -440,7 +429,6 @@ macro_rules! impl_object_methods {
         Ok(result)
       }
 
-      #[inline]
       pub fn get_property_names(&self) -> Result<JsObject> {
         let mut raw_value = ptr::null_mut();
         check_status!(unsafe {
@@ -449,10 +437,9 @@ macro_rules! impl_object_methods {
         Ok(unsafe { JsObject::from_raw_unchecked(self.0.env, raw_value) })
       }
 
-      /// https://nodejs.org/api/n-api.html#n_api_napi_get_all_property_names
+      /// <https://nodejs.org/api/n-api.html#n_api_napi_get_all_property_names>
       /// return `Array` of property names
       #[cfg(feature = "napi6")]
-      #[inline]
       pub fn get_all_property_names(
         &self,
         mode: KeyCollectionMode,
@@ -474,7 +461,6 @@ macro_rules! impl_object_methods {
       }
 
       /// This returns the equivalent of `Object.getPrototypeOf` (which is not the same as the function's prototype property).
-      #[inline]
       pub fn get_prototype<T>(&self) -> Result<T>
       where
         T: NapiValue,
@@ -484,7 +470,6 @@ macro_rules! impl_object_methods {
         unsafe { T::from_raw(self.0.env, result) }
       }
 
-      #[inline]
       pub fn get_prototype_unchecked<T>(&self) -> Result<T>
       where
         T: NapiValue,
@@ -494,7 +479,6 @@ macro_rules! impl_object_methods {
         Ok(unsafe { T::from_raw_unchecked(self.0.env, result) })
       }
 
-      #[inline]
       pub fn set_element<T>(&mut self, index: u32, value: T) -> Result<()>
       where
         T: NapiRaw,
@@ -504,7 +488,6 @@ macro_rules! impl_object_methods {
         })
       }
 
-      #[inline]
       pub fn has_element(&self, index: u32) -> Result<bool> {
         let mut result = false;
         check_status!(unsafe {
@@ -513,7 +496,6 @@ macro_rules! impl_object_methods {
         Ok(result)
       }
 
-      #[inline]
       pub fn delete_element(&mut self, index: u32) -> Result<bool> {
         let mut result = false;
         check_status!(unsafe {
@@ -522,7 +504,6 @@ macro_rules! impl_object_methods {
         Ok(result)
       }
 
-      #[inline]
       pub fn get_element<T>(&self, index: u32) -> Result<T>
       where
         T: NapiValue,
@@ -534,7 +515,6 @@ macro_rules! impl_object_methods {
         unsafe { T::from_raw(self.0.env, raw_value) }
       }
 
-      #[inline]
       pub fn get_element_unchecked<T>(&self, index: u32) -> Result<T>
       where
         T: NapiValue,
@@ -547,7 +527,6 @@ macro_rules! impl_object_methods {
       }
 
       /// This method allows the efficient definition of multiple properties on a given object.
-      #[inline]
       pub fn define_properties(&mut self, properties: &[Property]) -> Result<()> {
         check_status!(unsafe {
           sys::napi_define_properties(
@@ -565,7 +544,6 @@ macro_rules! impl_object_methods {
 
       /// Perform `is_array` check before get the length
       /// if `Object` is not array, `ArrayExpected` error returned
-      #[inline]
       pub fn get_array_length(&self) -> Result<u32> {
         if self.is_array()? != true {
           return Err(Error::new(
@@ -577,7 +555,6 @@ macro_rules! impl_object_methods {
       }
 
       /// use this API if you can ensure this `Object` is `Array`
-      #[inline]
       pub fn get_array_length_unchecked(&self) -> Result<u32> {
         let mut length: u32 = 0;
         check_status!(unsafe {
@@ -585,6 +562,16 @@ macro_rules! impl_object_methods {
         })?;
         Ok(length)
       }
+
+      #[cfg(feature = "napi8")]
+      pub fn freeze(&mut self) -> Result<()> {
+        check_status!(unsafe { sys::napi_object_freeze(self.0.env, self.0.value) })
+      }
+
+      #[cfg(feature = "napi8")]
+      pub fn seal(&mut self) -> Result<()> {
+        check_status!(unsafe { sys::napi_object_seal(self.0.env, self.0.value) })
+      }
     }
   };
 }
@@ -649,7 +636,6 @@ impl_napi_value_trait!(JsExternal, External);
 impl_napi_value_trait!(JsSymbol, Symbol);
 
 impl NapiValue for JsUnknown {
-  #[inline]
   unsafe fn from_raw(env: sys::napi_env, value: sys::napi_value) -> Result<Self> {
     Ok(JsUnknown(Value {
       env,
@@ -658,7 +644,6 @@ impl NapiValue for JsUnknown {
     }))
   }
 
-  #[inline]
   unsafe fn from_raw_unchecked(env: sys::napi_env, value: sys::napi_value) -> Self {
     JsUnknown(Value {
       env,
@@ -670,7 +655,6 @@ impl NapiValue for JsUnknown {
 
 impl NapiRaw for JsUnknown {
   /// get raw js value ptr
-  #[inline]
   unsafe fn raw(&self) -> sys::napi_value {
     self.0.value
   }
@@ -678,19 +662,16 @@ impl NapiRaw for JsUnknown {
 
 impl<'env> NapiRaw for &'env JsUnknown {
   /// get raw js value ptr
-  #[inline]
   unsafe fn raw(&self) -> sys::napi_value {
     self.0.value
   }
 }
 
 impl JsUnknown {
-  #[inline]
   pub fn get_type(&self) -> Result<ValueType> {
     unsafe { type_of!(self.0.env, self.0.value) }
   }
 
-  #[inline]
   /// # Safety
   ///
   /// This function should be called after `JsUnknown::get_type`
